//+------------------------------------------------------------------+
//|                                                    StepMA_v7.mq5 |
//|                        Copyright 2011, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2011, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"
#property indicator_chart_window

#property indicator_buffers 6
#property indicator_color1 Blue
//#property indicator_color2 Blue
//#property indicator_color3 Red
#property indicator_plots  1
extern int     Length     =10;      // Volty Length
extern double  Kv         =1.0;     // Sensivity Factor
extern int     StepSize   =0;       // Constant Step Size (if need)
extern int     MA_Mode    =0;       // Volty MA Mode : 0-SMA, 1-LWMA 
extern int     Advance    =0;       // Offset
extern double  Percentage =0;       // Percentage of Up/Down Moving   
extern bool    HighLow    =false;   // High/Low Mode Switch (more sensitive)
extern int     ColorMode  =0;       // Color Mode Switch
extern int     BarsNumber =0;

double LineBuffer[];
double UpBuffer[];
double DnBuffer[];
double smin[];
double smax[];
double trend[];
double StepMA=0, ATR0=0,ATRmax=-100000,ATRmin=1000000;
int limit;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
   string short_name;
//---- indicator line
   SetIndexBuffer(0,LineBuffer, INDICATOR_DATA);
   SetIndexBuffer(1,UpBuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(2,DnBuffer,INDICATOR_CALCULATIONS);
   //
   SetIndexBuffer(3,smin,INDICATOR_CALCULATIONS);
   SetIndexBuffer(4,smax,INDICATOR_CALCULATIONS);
   SetIndexBuffer(5,trend,INDICATOR_CALCULATIONS);
//---- name for DataWindow and indicator subwindow label
   short_name="StepMA("+Length+","+Kv+","+StepSize+")";
   IndicatorSetString(INDICATOR_SHORTNAME,short_name);
   
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,Length-1);
   //PlotIndexSetInteger(1,PLOT_DRAW_BEGIN,Length-1);
   //PlotIndexSetInteger(2,PLOT_DRAW_BEGIN,Length-1);
   
//---
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---
      int shift;
//----
   int Nbars = rates_total;
   if(prev_calculated > 0) limit=Nbars-prev_calculated;
   if(prev_calculated == 0) limit=Nbars-Length-1;
//----
   for(shift=limit;shift>=0;shift--)
     {
      int Step=StepSizeCalc( Length, Kv, StepSize, shift, high, low);
      //Comment (" StepSize= ", Step);
      StepMA=StepMACalc(HighLow, Step, shift, close, high, low, prev_calculated)+Percentage/100.0*Step*Point();
      if(ColorMode==0) LineBuffer[shift]=StepMA;
      if(ColorMode==1)
        {
         if(trend[shift]>0){UpBuffer[shift]=StepMA-Step*Point();DnBuffer[shift]=EMPTY_VALUE;}
         else
            if(trend[shift]<0){DnBuffer[shift]=StepMA+Step*Point();UpBuffer[shift]=EMPTY_VALUE;}
        }
      else
         if(ColorMode==2)
           {
            if (trend[shift]>0)
              {
               UpBuffer[shift]=StepMA;
               if(trend[shift+1] < 0)UpBuffer[shift+1]=DnBuffer[shift+1];
               DnBuffer[shift]=EMPTY_VALUE;
              }
            if (trend[shift]<0)
              {
               DnBuffer[shift]=StepMA;
               if(trend[shift+1] > 0)DnBuffer[shift+1]=UpBuffer[shift+1];
               UpBuffer[shift]=EMPTY_VALUE;
              }
           }
         else
           {
            UpBuffer[shift]=EMPTY_VALUE; DnBuffer[shift]=EMPTY_VALUE;
           }
     }
   
//--- return value of prev_calculated for next call
   return(rates_total);
  }
  
  //+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
   double StepSizeCalc(int Len, double Km, int Size, int k,const double &AH[], const double &AL[])
  {
   double result;
   double alfa;
   if(Size==0)
     {
      double AvgRange=0;
      for(int i=Len-1;i>=0;i--)
        {
         if(MA_Mode==0)
            alfa= 1.0;
         else
            alfa= 1.0+1.0*(Len-i)/Len;
         AvgRange+= alfa*(AH[k+i]-AL[k+i]);
        }
      ATR0=AvgRange/Len;
//----
      if (ATR0>ATRmax) ATRmax=ATR0;
      if (ATR0<ATRmin) ATRmin=ATR0;
//----
      result=MathRound(0.5*Km*(ATRmax+ATRmin)/Point());
     }
   else
      result=Km*StepSize;
   return(result);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
   double StepMACalc (bool HL, double Size, int k, const double &AC[], const double &AH[], const double &AL[], int pc)
  {
   double result;
//----
   if (HL)
     {
      smax[k]=AL[k]+2.0*Size*Point();
      smin[k]=AH[k]-2.0*Size*Point();
     }
   else
     {
      smax[k]=AC[k]+2.0*Size*Point();
      smin[k]=AC[k]-2.0*Size*Point();
     }
   if (pc==0){smax[limit+1]=smax[limit];smin[limit+1]=smin[limit];trend[limit+1]=0;}
//----
   trend[k]=trend[k+1];
   if (AC[k]>smax[k+1]) trend[k]=1;
   if (AC[k]<smin[k+1]) trend[k]=-1;
   if(trend[k]>0)
     {
      if(smin[k]<smin[k+1]) smin[k]=smin[k+1];
      result=smin[k]+Size*Point();
     }
   else
     {
      if(smax[k]>smax[k+1]) smax[k]=smax[k+1];
      result=smax[k]-Size*Point();
     }
   //Print (" k=",k," trend=",trend[k], " res=",result," Smax=", smax[k], " Smin=", smin[k]);
   return(result);
  }

//+------------------------------------------------------------------+
